﻿using System;
using System.Data;
using System.Data.OleDb;
using System.Windows.Forms;

namespace DbNavigator
{
    public partial class Form1 : Form
    {
        readonly string TABLE_NAME = "Order";
        const int ROWS_PER_PAGE = 100;
        int CURRENT_PAGE = 1;
        int DB_INDEX = 0;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            SetButtons();
            UpdateGrid();
        }

        /// <summary>
        /// Create toolstrip page buttons according to number of rows
        /// </summary>
        private void SetButtons()
        {
            toolStrip1.Items.Add(new ToolStripLabel("Pages : "));
            int numberOfRecords = GetNumberOfRecordsInTable(TABLE_NAME);
            double pageCount = Math.Ceiling(numberOfRecords * 1.0f / ROWS_PER_PAGE);
            for (int i = 1; i <= pageCount; i++)
            {
                toolStrip1.Items.Add(new ToolStripButton($"{i}", null, OnClick, $"{i}"));
            }
        }

        private void OnClick(object sender, EventArgs e)
        {
            //update the current page with the clicked button
            CURRENT_PAGE = Convert.ToInt32((sender as ToolStripButton).Name);

            //update database index according to current page
            DB_INDEX = (CURRENT_PAGE - 1) * ROWS_PER_PAGE;

            //Update Data in Grid after updating the index values
            UpdateGrid();
        }

        /// <summary>
        /// Gets total number of records in the table
        /// </summary>
        private int GetNumberOfRecordsInTable(string tableName)
        {
            int numberOfRecords = 0;
            using (OleDbConnection conn = new OleDbConnection(GetConnectionString()))
            {
                DataTable dt = new DataTable();
                string sql = $"SELECT Count(*) FROM [{tableName}]";
                OleDbDataAdapter adapter = new OleDbDataAdapter(sql, conn);
                adapter.Fill(dt);
                numberOfRecords = (int)dt.Rows[0].ItemArray[0];
            }
            return numberOfRecords;
        }

        /// <summary>
        /// Gets data from database according to page number and updates the Grid
        /// </summary>
        public void UpdateGrid()
        {
            using (OleDbConnection conn = new OleDbConnection(GetConnectionString()))
            {
                string sql = $"SELECT * FROM [{TABLE_NAME}]";
                OleDbDataAdapter pagingAdapter = new OleDbDataAdapter(sql, conn);
                DataSet pagingDS = new DataSet();

                //Get limited data from database
                pagingAdapter.Fill(pagingDS, DB_INDEX, ROWS_PER_PAGE, TABLE_NAME);

                c1FlexGrid1.BeginUpdate();

                c1FlexGrid1.DataSource = pagingDS;
                c1FlexGrid1.DataMember = TABLE_NAME;

                c1FlexGrid1.EndUpdate();
            }
        }

        /// <summary>
        /// Gets Connection string
        /// </summary>
        static string GetConnectionString()
        {
            string path = Environment.GetFolderPath(Environment.SpecialFolder.Personal) + @"\ComponentOne Samples\Common";
            string conn = @"provider=microsoft.jet.oledb.4.0;data source={0}\c1nwind.mdb;";
            return string.Format(conn, path);
        }
    }
}
